@@ -48,16 +48,22 @@ struct Score {
 };
 
 struct ScoreCard {
+    QStringView search;
+    QString term;
     Bitap::Match bitap;
     qreal bitapScore;
     int levenshtein;
     qreal levenshteinScore;
+    bool perfectMatch;
+    bool completeMatch;
+    bool startsWith;
 };
 
 QDebug operator<<(QDebug dbg, const ScoreCard &card)
 {
-    dbg.nospace() << "Scorecard(" << "bitap: " << card.bitap << ", bitapScore: " << card.bitapScore << ", levenshtein: " << card.levenshtein
-                    << ", levenshteinScore: " << card.levenshteinScore << ")";
+    dbg.nospace() << "Scorecard(" << "search: " << card.search << ", term: " << card.term << "bitap: " << card.bitap << ", bitapScore: " << card.bitapScore
+                  << ", levenshtein: " << card.levenshtein << ", levenshteinScore: " << card.levenshteinScore << ", perfectMatch: " << card.perfectMatch
+                  << ", completeMatch: " << card.completeMatch << ", startsWith: " << card.startsWith << ")";
     return dbg;
 }
 
@@ -102,21 +108,26 @@ auto makeScores(const auto &notNormalizedString, const auto &queryList) {
 
         // Mind that we give different levels of bonus. This is important to imply ordering within competing matches of the same "type".
         // If we perfectly match that gives a bonus for not requiring any changes.
-        const auto noSubstitionBonus = Bitap::score(string, bitap.value(), 0) == 1.0 ? 4.0 : 1.0;
+        const auto perfectMatch = bitap.value().distance == 0;
         // If we match the entire length of the string that gets a bonus (disregarding distance, that was considered above).
-        const auto completeMatchBonus = bitap->end >= (queryItem.size() - 1) ? 3.0 : 1.0;
+        const auto completeMatch = bitap->size >= (queryItem.size());
         // If the string starts with the query item that gets a bonus.
-        const auto startsWithBonus = (string.startsWith(queryItem, Qt::CaseInsensitive)) ? 2.0 : 1.0;
+        const auto startsWith = (string.startsWith(queryItem, Qt::CaseInsensitive));
 
         // Also consider the distance between the input and the query item.
         // If one is "yolotrollingservice" and the other is "yolo" then we must consider them worse matches than say "yolotroll".
         const auto levenshtein = Levenshtein::distance(string, queryItem);
 
         cards.emplace_back(ScoreCard{
+            .search = queryItem,
+            .term = string,
             .bitap = *bitap,
-            .bitapScore = bitapScore + completeMatchBonus + noSubstitionBonus + startsWithBonus,
+            .bitapScore = bitapScore + (perfectMatch ? 4.0 : 0.0) + (completeMatch ? 3.0 : 0.0) + (startsWith ? 2.0 : 0.0),
             .levenshtein = levenshtein,
             .levenshteinScore = Levenshtein::score(string, levenshtein),
+            .perfectMatch = perfectMatch,
+            .completeMatch = completeMatch,
+            .startsWith = startsWith,
         });
     }
 
@@ -309,11 +320,11 @@ private:
             return Score{.value = std::numeric_limits<decltype(Score::value)>::max(), .categoryRelevance = KRunner::QueryMatch::CategoryRelevance::Highest};
         }
 
-        std::array<WeightedScoreCard, 4> weightedCards = {
-            WeightedScoreCard{.cards = makeScores(name, queryList), .weight = 1.0},
-            WeightedScoreCard{.cards = makeScores(service->untranslatedName(), queryList), .weight = 0.8},
-            WeightedScoreCard{.cards = makeScores(service->genericName(), queryList), .weight = 0.6},
-            WeightedScoreCard{.cards = makeScoreFromList(queryList, service->keywords()), .weight = 0.1},
+        const auto weightedCards = {
+            WeightedScoreCard{.cards = makeScores(name, queryList), .weight = 100.0},
+            WeightedScoreCard{.cards = makeScores(service->untranslatedName(), queryList), .weight = 75.0},
+            WeightedScoreCard{.cards = makeScores(service->genericName(), queryList), .weight = 50.0},
+            WeightedScoreCard{.cards = makeScoreFromList(queryList, service->keywords()), .weight = 25.0},
         };
 
         if (RUNNER_SERVICES().isDebugEnabled()) {
@@ -324,21 +335,32 @@ private:
             qCDebug(RUNNER_SERVICES) << "-------";
         }
 
-        int scores = 1; // starts at 1 to avoid division by zero
+        int scores = 1; // starts at 1 to avoid division by zero. Is the number of scores to average over.
         qreal finalScore = 0.0;
-        for (const auto &weightedCard : weightedCards) {
-            if (weightedCard.cards.empty()) {
-                continue; // No scores, no match.
-            }
 
-            qreal weightedScore = 0.0;
-            for (const auto &scoreCard : weightedCard.cards) {
-                weightedScore += (scoreCard.bitapScore + scoreCard.levenshteinScore) * weightedCard.weight;
+        const auto perfectMatchScore = [&](const auto &weightedCards) {
+            if (std::ranges::any_of(weightedCards.cards, [](const ScoreCard &card) {
+                    return card.perfectMatch;
+                })) {
+                finalScore += 100.0 * weightedCards.weight;
                 scores++;
             }
+        };
 
+        const auto fuzzyScore = [&](const auto &weightedCards) {
+            qreal weightedScore = 0.0;
+            for (const auto &scoreCard : weightedCards.cards) {
+                weightedScore += (scoreCard.bitapScore + scoreCard.levenshteinScore) * weightedCards.weight;
+                scores++;
+            }
             finalScore += weightedScore;
+        };
+
+        for (const auto &weightedCard : weightedCards) {
+            perfectMatchScore(weightedCard);
+            fuzzyScore(weightedCard);
         }
+
         finalScore = finalScore / scores; // Average the score for this card
 
         qCDebug(RUNNER_SERVICES) << "Final score for" << name << "is" << finalScore;
@@ -366,7 +388,12 @@ private:
 
             setupMatch(service, match);
             match.setCategoryRelevance(score->categoryRelevance);
-            match.setRelevance(score->value);
+            // KRunner may apply counter-productive bumps to the score of up to 0.5 points. That can easily produce
+            // unrealistic results where suddenly one thing is top score for no discernable reason. We outscore KRunner
+            // by moving our score range into the hundreds, making the 0.5 bump negligible.
+            // In Plasma 6.6 and later we'll depend on a KRunner that does no longer have this behavior.
+            constexpr qreal outscoreMultiplier = 100.0;
+            match.setRelevance(score->value * outscoreMultiplier);
             qCDebug(RUNNER_SERVICES) << match.text() << "is this relevant:" << match.relevance() << "category relevance" << match.categoryRelevance();
 
             matches << match;
